import { VueConstructor } from 'vue';
import VueCompositionAPI from '@vue/composition-api';
import * as components from './components';

interface VueConstructorWithCompositionAPI extends VueConstructor {
  __composition_api_installed__?: boolean
}

export let _Vue:VueConstructorWithCompositionAPI;
export let _CompositionApi: any;
/**
 * Install all components
 *
 * @param  {Vue}  Vue The Vue object
 * @return {void}
 */

export default (Vue:VueConstructorWithCompositionAPI) => {
  _Vue = Vue;
  if(!Vue.__composition_api_installed__){
    Vue.use(VueCompositionAPI);
  }    
  else{    
    throw new Error('Currently Apps using composition API are not supported');
  }

  Object.keys(components).forEach((name) => {
    Vue.component(name, (components as any)[name]);
  });
};